# Story 1.2: Interactive Wizard for Core Project Details

**Status:** Draft

## Goal & Context

**User Story:** As a Python developer, I want an interactive wizard after running `pyhatchery new <project_name>` (and after initial name validation) to easily provide essential project details.

**Context:** This story implements the interactive mode for gathering project metadata (PRD FR1). It follows Story 1.1 (CLI command) and Story 1.1A (name validation). The collected data will be used to populate the `project_context` for file templating in Epic 2.

## Detailed Requirements

- The wizard prompts for:
  - Author Name (default: from `git config user.name`, else empty)
  - Author Email (default: from `git config user.email`, else empty)
  - GitHub Username (default: empty)
  - Project Description (default: empty)
  - License (default: "MIT"; provide a small list of common choices e.g., MIT, Apache-2.0, GPL-3.0)
  - Python Version Preference for the generated project (default: suggest a recent stable version like 3.10 or 3.11 from a predefined list).
- Each prompt should clearly indicate what information is needed.
- User input should be captured for personalization.
- The wizard should only proceed after the project name has been captured and validated (as per Story 1.1 and 1.1A), potentially offering a chance to re-input the project name if warnings occurred.

## Acceptance Criteria (ACs)

- AC1: After `pyhatchery new my_project` and successful name validation (Story 1.1A), the wizard prompts for Author Name, Author Email, GitHub Username, Description, License, and Python Version.
- AC2: Defaults for Author Name and Email are correctly pre-filled if available in git config.
- AC3: User can navigate through prompts and provide input for all required fields.
- AC4: All inputs provided through the wizard are correctly captured.
- AC5: User can accept default values by pressing Enter (if applicable for the input library chosen).

## Technical Implementation Context

**Guidance:** Use the following details for implementation. Refer to the linked `docs/` files for broader context if needed.

- **Relevant Files:**
  - Files to Create: `src/pyhatchery/components/interactive_wizard.py`, `src/pyhatchery/components/config_loader.py` (for git config access)
  - Files to Modify: `src/pyhatchery/cli.py` (to invoke the wizard in interactive mode).
  - _(Hint: See `docs/project-structure.md` for component suggestions)_

- **Key Technologies:**
  - Python `>=3.11`
  - `input()` built-in function for basic prompts, or a library like `questionary` or `click` for richer interactions (if desired, but `input()` is MVP).
  - `subprocess` module (to call `git config user.name` and `git config user.email`).
  - _(Hint: See `docs/tech-stack.md` if available)_

- **API Interactions / SDK Usage:**
  - `git config user.name`
  - `git config user.email`
  - _(Hint: `subprocess.run` can be used to capture output. Handle cases where git is not installed or config is not set.)_

- **Data Structures:**
  - The collected details (author name, email, github username, description, license, python version) will be stored and added to the `project_context` object.
  - License choices: `["MIT", "Apache-2.0", "GPL-3.0"]` (or as defined in PRD).
  - Python version choices: e.g., `["3.10", "3.11", "3.12"]` (configurable, use a recent stable default).
  - _(Hint: See `docs/data-models.md` for `project_context`)_

- **Environment Variables:**
  - Not directly used.
  - _(Hint: See `docs/environment-vars.md`)_

- **Coding Standards Notes:**
  - `interactive_wizard.py` should encapsulate all wizard logic (prompting, input gathering).
  - `config_loader.py` should handle fetching defaults from `git config`. Make it fault-tolerant (e.g., if `git` command fails or config is not set).
  - Prompts should be clear and user-friendly.
  - Implement the "re-enter project name or proceed" logic if warnings from Story 1.1A occurred. This may involve a loop or conditional flow in `cli.py` or the main orchestrator.
  - _(Hint: See `docs/coding-standards.md` for style and error handling)_

## Tasks / Subtasks

- [ ] Create `src/pyhatchery/components/config_loader.py`:
  - [ ] Implement `get_git_config_value(key: str) -> str | None` function using `subprocess` to fetch git config values (e.g., `user.name`, `user.email`). Handle errors gracefully (e.g., git not found, key not set).
- [ ] Create `src/pyhatchery/components/interactive_wizard.py`:
  - [ ] Implement a main function `collect_project_details(project_name: str, name_warnings: list) -> dict`:
    - [ ] If `name_warnings` (from Story 1.1A) exist, display them and ask user if they want to proceed with `project_name` or re-enter. If re-enter, this function might need to signal back to CLI to restart name input, or handle re-input itself (simpler for now: assume CLI handles loop, wizard just gets current name). For MVP, wizard can just ask "Proceed with this name (y/n)?" if warnings. If 'n', tool exits or signals re-entry.
    - [ ] Prompt for Author Name, using `config_loader.get_git_config_value("user.name")` as default.
    - [ ] Prompt for Author Email, using `config_loader.get_git_config_value("user.email")` as default.
    - [ ] Prompt for GitHub Username (no default initially, or could try to infer from email if complex).
    - [ ] Prompt for Project Description.
    - [ ] Prompt for License choice from a predefined list (e.g., "MIT", "Apache-2.0", "GPL-3.0"), with "MIT" as default.
    - [ ] Prompt for Python Version preference from a predefined list (e.g., "3.10", "3.11", "3.12"), with a sensible default (e.g., "3.11").
    - [ ] Collect all inputs into a dictionary.
- [ ] In `src/pyhatchery/cli.py`:
  - [ ] If not in non-interactive mode (see Story 1.3), after project name capture (Story 1.1) and validation (Story 1.1A), call `interactive_wizard.collect_project_details()`.
  - [ ] Store the returned dictionary of details (for `project_context`).
- [ ] Ensure user can accept defaults by pressing Enter (if using `input()`, this means checking for empty string and using default).

## Testing Requirements

**Guidance:** Verify implementation against the ACs using the following tests.

- **Unit Tests:**
  - `test_config_loader.py`:
    - Mock `subprocess.run` to simulate:
      - `git config` returning a value.
      - `git config` key not set.
      - `git` command not found.
  - `test_interactive_wizard.py`:
    - Mock `input()` (and `config_loader` calls) to simulate user providing all values.
    - Mock `input()` to simulate user accepting all defaults.
    - Test prompt flow and data capture for each field.
    - Test license selection and Python version selection logic.
    - Test handling of "proceed or re-enter name" if warnings from 1.1A are passed in (AC4 from 1.1A).
  - Location: `tests/unit/components/`
- **Integration Tests:**
  - In `tests/integration/test_project_generation.py`:
    - Difficult to test full interactive CLI with `subprocess` without advanced PTY/pexpect.
    - Focus on testing the wizard's data collection logic when called directly if CLI invocation is too complex.
    - Alternatively, test the CLI with mocked `interactive_wizard.collect_project_details` to ensure it's called correctly in interactive mode.
- _(Hint: See `docs/testing-strategy.md`. Interactive CLI testing can be challenging; prioritize unit tests for wizard logic.)_
