# Story 1.3: Non-Interactive Mode with CLI Flags and `.env` Support

**Status:** Draft

## Goal & Context

**User Story:** As a Python developer, I want to generate a project non-interactively by providing all details via CLI flags or an `.env` file for automation and consistency.

**Context:** This story implements the non-interactive mode for project generation (PRD FR2), providing an alternative to the wizard (Story 1.2). It allows for scripted or automated project creation. It builds upon Story 1.1 (CLI command) and Story 1.1A (name validation).

## Detailed Requirements

- Implement CLI flags for all details gathered by the wizard: `--author`, `--email`, `--github-username`, `--description`, `--license`, `--python-version`.
- Add a `--no-interactive` flag to suppress the wizard and rely on CLI flags or `.env` file.
- Project name validation (Story 1.1A) must still be performed; warnings will be displayed, but in non-interactive mode, the process continues without prompting to re-enter (unless a fatal error like output dir exists).
- If `--no-interactive` is used and required details are missing from flags, the tool should attempt to load them from a `.env` file in the current directory.
- If required details are still missing in non-interactive mode, display an error message listing the missing parameters and exit.
- CLI flags should override `.env` values, which override defaults.

## Acceptance Criteria (ACs)

- AC1: Running `pyhatchery new my_project --no-interactive --author "Test User" --email "test@example.com" ...` generates a project without prompts (but will show PyPI/PEP8 warnings if applicable).
- AC2: If a `.env` file exists with `AUTHOR_NAME="Env User"`, and `pyhatchery new my_project --no-interactive` is run without an `--author` flag, "Env User" is used.
- AC3: If `--no-interactive` is used and a required field (e.g., author name, if no default mechanism and not in .env) is not provided via flag or `.env`, an informative error is shown and the tool exits.
- AC4: CLI flags correctly override values from a `.env` file.

## Technical Implementation Context

**Guidance:** Use the following details for implementation. Refer to the linked `docs/` files for broader context if needed.

- **Relevant Files:**
  - Files to Modify: `src/pyhatchery/cli.py` (to add new CLI flags and logic for non-interactive mode), `src/pyhatchery/components/config_loader.py` (to add `.env` loading).
  - Files to Create: `.env.example` in the project root.
  - _(Hint: See `docs/project-structure.md` and `docs/environment-vars.md` regarding `.env` usage for _inputs_)_

- **Key Technologies:**
  - Python `>=3.11`
  - `argparse` (for adding new CLI flags).
  - `python-dotenv` library (for loading `.env` files).
  - _(Hint: PRD mentions `python-dotenv` as a runtime dependency)_

- **API Interactions / SDK Usage:**
  - Not applicable for this story.

- **Data Structures:**
  - CLI arguments will be parsed into a namespace or dictionary.
  - `.env` file variables (e.g., `AUTHOR_NAME`, `PROJECT_DESCRIPTION`) will be loaded.
  - Logic for merging these sources: CLI flags > `.env` values > defaults (from wizard logic or hardcoded).
  - _(Hint: See `docs/data-models.md` for `project_context` which these values populate)_

- **Environment Variables:**
  - The story involves reading from a `.env` file, not system environment variables.
  - Create an `.env.example` showing keys like `AUTHOR_NAME`, `AUTHOR_EMAIL`, `GITHUB_USERNAME`, `PROJECT_DESCRIPTION`, `LICENSE`, `PYTHON_VERSION`.
  - _(Hint: See `docs/environment-vars.md` for clarification on `.env` usage for _generation inputs_)_

- **Coding Standards Notes:**
  - Update `argparse` setup in `cli.py` to include the new flags.
  - In `config_loader.py`, add a function `load_env_vars(env_path: Path) -> dict` using `python-dotenv`.
  - The CLI logic needs to:
    - Check for `--no-interactive`.
    - If set, load CLI flags.
    - If flags are missing for required fields, attempt to load from `.env`.
    - Merge sources with correct precedence.
    - Validate that all required fields are present; if not, error out (AC3).
  - _(Hint: See `docs/coding-standards.md`)_

## Tasks / Subtasks

- [ ] Add `python-dotenv` to `[project.dependencies]` in `pyproject.toml`.
- [ ] Create `.env.example` file in the PyHatchery project root, listing expected variables (e.g., `AUTHOR_NAME`, `AUTHOR_EMAIL`, etc.).
- [ ] In `src/pyhatchery/components/config_loader.py`:
  - [ ] Add function `load_from_env(env_file_path: str = ".env") -> dict` to load variables from a `.env` file using `python-dotenv`. Return an empty dict if file not found.
- [ ] In `src/pyhatchery/cli.py`:
  - [ ] Add the `--no-interactive` boolean flag to `argparse`.
  - [ ] Add CLI flags for `--author`, `--email`, `--github-username`, `--description`, `--license`, `--python-version`. Define their `dest` to match keys in `project_context` or wizard outputs.
  - [ ] Modify the main CLI logic:
    - If `--no-interactive` is present:
      - Call `config_loader.load_from_env()` to get values from `.env`.
      - Get values from CLI arguments.
      - Implement merge logic: CLI args override `.env` values, which override hardcoded defaults (e.g., default license "MIT", default Python version).
      - Perform name validation (Story 1.1A) and display warnings, but do not prompt for re-entry.
      - Check if all required fields (project_name, author_name, author_email, etc. - define which are strictly required) are populated. If not, print error and exit (AC3).
    - Else (interactive mode):
      - Proceed with wizard (Story 1.2).
- [ ] Ensure CLI flags correctly override `.env` values (AC4).
- [ ] Ensure basic non-interactive generation works end-to-end for data collection (AC1).

## Testing Requirements

**Guidance:** Verify implementation against the ACs using the following tests.

- **Unit Tests:**
  - `test_config_loader.py`:
    - Test `load_from_env` with an existing `.env` file with values.
    - Test `load_from_env` with a non-existent `.env` file.
  - `test_cli.py`:
    - Test argument parsing for all new flags.
    - Test logic for merging contexts (CLI args, .env, defaults) with various combinations to check precedence.
    - Test error exit if required arguments are missing in non-interactive mode.
  - Location: `tests/unit/`
- **Integration Tests:** (Considered E2E/Acceptance for CLI)
  - In `tests/integration/test_project_generation.py`:
    - Create a temporary `.env` file with some values. Run `pyhatchery new my_project --no-interactive --author "CLI Author"` (where `.env` has a different author). Verify "CLI Author" is used.
    - Run `pyhatchery new my_project --no-interactive` with a `.env` file providing all details. Verify successful context population (internal check).
    - Run `pyhatchery new my_project --no-interactive` _without_ a required field (e.g., author, if made mandatory and no default) in flags or `.env`. Verify error message and non-zero exit code.
- _(Hint: See `docs/testing-strategy.md`)_
