# Story 1.4: Basic Project Directory Structure Creation

**Status:** Draft

## Goal & Context

**User Story:** As a Python developer, I want PyHatchery to create a standard, best-practice directory structure for my new project.

**Context:** This story implements the actual file system operation of creating the initial directories for the generated project (PRD FR4). It follows the gathering of project name and details (Stories 1.1-1.3). The created structure will be populated with files in Epic 2.

## Detailed Requirements

- Create a root directory named `project_name` (original user input, can have spaces/caps – this will be the directory specified by `--output-dir` in Story 1.5, or CWD if not specified. The actual name of this root dir should be based on original `project_name`).
- Inside the root, create:
  - `src/<project_name_slug>/` (where `project_name_slug` is the Python module-friendly version like "my_project_name").
  - `tests/`
  - `docs/` (can be empty initially)

## Acceptance Criteria (ACs)

- AC1: After successful execution, a root directory matching `project_name` (original input, or slug if chosen for root dir name) is created in the target location (CWD or `--output-dir`).
- AC2: The `src/<project_name_slug>/`, `tests/`, and `docs/` subdirectories are created within the root.
- AC3: `project_name_slug` is correctly derived (e.g., "My Project" leads to `src/my_project/`).

## Technical Implementation Context

**Guidance:** Use the following details for implementation. Refer to the linked `docs/` files for broader context if needed.

- **Relevant Files:**
  - Files to Create: `src/pyhatchery/components/project_generator.py` (or similar for file system operations).
  - Files to Modify: `src/pyhatchery/cli.py` (or main orchestrator, to call the directory creation logic).
  - _(Hint: See `docs/project-structure.md` for component `project_generator.py`)_

- **Key Technologies:**
  - Python `>=3.11`
  - `pathlib` module (for path manipulation and directory creation, e.g., `Path.mkdir(parents=True, exist_ok=True)`).
  - _(Hint: See `docs/tech-stack.md` if available)_

- **API Interactions / SDK Usage:**
  - Not applicable.

- **Data Structures:**
  - Requires `project_name` (original user input, for the root directory name).
  - Requires `project_slug_python` (the underscore-separated version for `src/<project_slug_python>/`). This should come from the `project_context` populated by earlier steps (name service/slugification - Story 1.1A and Story 2.8).
  - _(Hint: See `docs/data-models.md` for `project_context` keys like `project_name` and `project_slug_python`)_

- **Environment Variables:**
  - Not applicable.
  - _(Hint: See `docs/environment-vars.md`)_

- **Coding Standards Notes:**
  - Encapsulate directory creation logic in `project_generator.py`.
  - Use `pathlib` for all path operations.
  - Handle potential `IOError` or `OSError` during directory creation, though `exist_ok=True` can prevent some if re-run. Story 1.5 will handle existing target project directory errors.
  - Ensure `project_slug_python` is correctly used for the `src/` subdirectory.
  - _(Hint: See `docs/coding-standards.md`)_

## Tasks / Subtasks

- [ ] Create `src/pyhatchery/components/project_generator.py`.
  - [ ] Implement a function `create_base_structure(output_path: Path, project_name_original: str, python_package_slug: str)`:
    - [ ] Determine the root project path: `output_path / project_name_original`. (Note: Story 1.5 will handle `--output-dir`. For now, `output_path` can be `Path.cwd()`).
    - [ ] Create the root project directory.
    - [ ] Create `root_project_path / "src" / python_package_slug`.
    - [ ] Create `root_project_path / "tests"`.
    - [ ] Create `root_project_path / "docs"`.
    - [ ] Use `Path.mkdir(parents=True, exist_ok=False)` to ensure it doesn't overwrite. Story 1.5 addresses pre-existence check. For this story, `exist_ok=True` might be simpler if 1.5 handles the top-level check. Let's assume Story 1.5 will ensure `output_path / project_name_original` is clear.
- [ ] In `src/pyhatchery/cli.py` (or main orchestrator):
  - [ ] After all project details are collected and confirmed (interactive or non-interactive), and after output directory is determined (Story 1.5 will refine this), call `project_generator.create_base_structure()`.
  - [ ] Pass the correct `project_name` (original for root dir) and `project_slug_python` (derived, for `src/` dir) to this function.
- [ ] Confirm `project_slug_python` is correctly derived and used (AC3). The actual derivation is part of Story 1.1A (initial) and refined in Story 2.8 (finalized). This story _uses_ the derived slug.

## Testing Requirements

**Guidance:** Verify implementation against the ACs using the following tests.

- **Unit Tests:**
  - `test_project_generator.py`:
    - Mock `pathlib.Path.mkdir` to verify it's called with correct paths for root, src, tests, docs.
    - Test the path construction logic within `create_base_structure`.
  - Location: `tests/unit/components/`
- **Integration Tests:** (Considered E2E/Acceptance for CLI)
  - In `tests/integration/test_project_generation.py`:
    - Run `pyhatchery new TestProject --no-interactive ...` (with minimal required flags).
    - Use `tmp_path` pytest fixture to create projects in a temporary directory.
    - Assert that the directory `TestProject/` is created.
    - Assert that `TestProject/src/test_project/` (assuming "test_project" is the Python slug), `TestProject/tests/`, and `TestProject/docs/` are created.
    - Clean up the created directories after the test.
- _(Hint: See `docs/testing-strategy.md` for E2E tests involving file system assertions)_
