# Story 1.5: Custom Output Location

**Status:** Draft

## Goal & Context

**User Story:** As a Python developer, I want to specify where PyHatchery creates the new project directory, instead of always using the current working directory.

**Context:** This story adds flexibility for the user to control the output location of the generated project (PRD FR3). It modifies the CLI (Story 1.1) and interacts with directory creation logic (Story 1.4).

## Detailed Requirements

- Implement an `--output-dir` (or `-o`) CLI flag.
- If provided, the root project directory should be created inside this specified path.
- If the output directory does not exist, attempt to create it.
- If the target project directory (`<output_dir>/<project_name>`) already exists, the tool should warn the user and exit to prevent accidental overwrites.

## Acceptance Criteria (ACs)

- AC1: Running `pyhatchery new my_project --output-dir /tmp/projects` creates `my_project` inside `/tmp/projects/`.
- AC2: If `/tmp/projects` (the `--output-dir`) does not exist, it is created.
- AC3: If `/tmp/projects/my_project` (the target project path) already exists, an error message is displayed, and no files are overwritten, and the tool exits.
- AC4: If `--output-dir` is not provided, project is created in the current working directory.

## Technical Implementation Context

**Guidance:** Use the following details for implementation. Refer to the linked `docs/` files for broader context if needed.

- **Relevant Files:**
  - Files to Modify: `src/pyhatchery/cli.py` (add flag, pass output dir to generator), `src/pyhatchery/components/project_generator.py` (handle custom output dir, check for existing project).
  - _(Hint: See `docs/project-structure.md`)_

- **Key Technologies:**
  - Python `>=3.11`
  - `argparse` (for CLI flag).
  - `pathlib` (for path operations, checking existence, creating directory).
  - _(Hint: See `docs/tech-stack.md` if available)_

- **API Interactions / SDK Usage:**
  - Not applicable.

- **Data Structures:**
  - `output_dir` (Path object).
  - `project_name` (string, for the final project directory name).
  - _(Hint: See `docs/data-models.md`)_

- **Environment Variables:**
  - Not applicable.
  - _(Hint: See `docs/environment-vars.md`)_

- **Coding Standards Notes:**
  - Update `argparse` in `cli.py`.
  - Logic in `project_generator.py` (or `cli.py` before calling generator) should:
    - Resolve the `output_dir` (default to `Path.cwd()`).
    - Construct the full target project path: `output_dir / project_name`.
    - Check if `output_dir` exists; if not, create it (AC2).
    - Check if `output_dir / project_name` exists; if yes, error and exit (AC3).
  - Ensure clear error messages.
  - _(Hint: See `docs/coding-standards.md`)_

## Tasks / Subtasks

- [ ] In `src/pyhatchery/cli.py`:
  - [ ] Add `--output-dir` / `-o` flag to `argparse`. It should store a `Path` object or string.
  - [ ] Determine the `final_output_dir`: if `--output-dir` is provided, use it, else use `Path.cwd()`.
  - [ ] Pass `final_output_dir` and `project_name` to the component responsible for directory creation.
- [ ] In `src/pyhatchery/components/project_generator.py` (or modify `create_base_structure` from Story 1.4):
  - [ ] The function (e.g., `setup_project_directory(base_output_dir: Path, project_name_original: str) -> Path`) should take the base output directory and original project name.
  - [ ] Construct `target_project_path = base_output_dir / project_name_original`.
  - [ ] If `base_output_dir` does not exist, create it using `base_output_dir.mkdir(parents=True, exist_ok=True)`.
  - [ ] Check if `target_project_path` already exists. If yes, print an error message and raise an exception (e.g., `FileExistsError`) to be caught by `cli.py` for graceful exit (AC3).
  - [ ] If it does not exist, this function can proceed to create `target_project_path` (as the root project dir for Story 1.4). Story 1.4's `create_base_structure` would then be called with `target_project_path` as its root.
  - [ ] Return `target_project_path` for subsequent operations.
- [ ] In `cli.py`, before calling `create_base_structure` (Story 1.4), call the new pre-check/setup logic for the output directory. Handle the potential `FileExistsError` and exit gracefully.

## Testing Requirements

**Guidance:** Verify implementation against the ACs using the following tests.

- **Unit Tests:**
  - `test_project_generator.py` (or wherever the logic resides):
    - Test directory creation logic with mocked `pathlib.Path` methods:
      - `output_dir` does not exist: verify it's created.
      - `target_project_path` already exists: verify error is raised/message logged.
      - `target_project_path` does not exist: verify successful path determination.
  - `test_cli.py`:
    - Test parsing of `--output-dir` flag.
    - Test default output directory (CWD) if flag is not provided.
  - Location: `tests/unit/`
- **Integration Tests:** (Considered E2E/Acceptance for CLI)
  - In `tests/integration/test_project_generation.py` (using `tmp_path`):
    - Run `pyhatchery new TestProject -o custom_output TestProjectArgs...`. Verify `custom_output/TestProject/` is created within `tmp_path`.
    - Create `tmp_path / "existing_dir" / "TestProject"` manually. Run `pyhatchery new TestProject -o existing_dir TestProjectArgs...`. Verify error message and that the tool exits without modifying `existing_dir/TestProject`.
    - Run `pyhatchery new TestProject TestProjectArgs...` (no `-o`). Verify `TestProject/` is created in the CWD (which would be a subdir of `tmp_path` for the test).
- _(Hint: See `docs/testing-strategy.md`)_
