# Story 1.6: Git Repository Initialization Guidance

**Status:** Draft

## Goal & Context

**User Story:** As a Python developer, after PyHatchery generates my project, I want clear instructions on how to initialize it as a new Git repository, ensuring no template history is included.

**Context:** This story provides the final user-facing instructions after successful project generation, focusing on Git setup (PRD FR8). It assumes PyHatchery's internal templates do not carry a `.git` history into the generated project.

## Detailed Requirements

- If PyHatchery uses an internal template that is itself a Git repository, ensure the `.git` directory from this template is *not* copied to the generated project. (This is an architectural consideration for PyHatchery's template management, not an action by this story's code, but a prerequisite).
- After project generation is complete (all files written, etc.), display a success message that includes concise instructions:

      ```
      Project <project_name> successfully hatched at <project_path>!
      Next steps:
        cd <project_path>
        git init
        git add .
        git commit -m "Initial commit: project structure from PyHatchery"
      ```

## Acceptance Criteria (ACs)

- AC1: The generated project directory does not contain a `.git` sub-directory from any template. (This is a check on the overall generation process, PyHatchery internal template design).
- AC2: A success message is displayed upon completion of all generation steps.
- AC3: The success message includes the recommended `git init`, `git add`, and `git commit` commands.
- AC4: The `<project_path>` in the message correctly points to the newly created project directory (considering CWD or `--output-dir` from Story 1.5 and the project name).

## Technical Implementation Context

**Guidance:** Use the following details for implementation. Refer to the linked `docs/` files for broader context if needed.

- **Relevant Files:**
  - Files to Modify: `src/pyhatchery/cli.py` (or the main orchestrator, to print the final success message).
  - *(Hint: See `docs/project-structure.md`)*

- **Key Technologies:**
  - Python `>=3.11`
  - Standard `print()` function.
  - *(Hint: See `docs/tech-stack.md` if available)*

- **API Interactions / SDK Usage:**
  - Not applicable.

- **Data Structures:**
  - Requires `project_name` (original user input).
  - Requires `final_project_path` (the absolute or relative path to the generated project root directory).
  - *(Hint: See `docs/data-models.md`)*

- **Environment Variables:**
  - Not applicable.
  - *(Hint: See `docs/environment-vars.md`)*

- **Coding Standards Notes:**
  - The message should be clear and formatted for readability.
  - Ensure `<project_name>` and `<project_path>` are dynamically inserted.
  - This message should be one of the last things printed by the CLI upon successful completion.
  - For AC1: While not implemented by this story, the development of PyHatchery's internal templating mechanism must ensure template `.git` dirs are not copied.
  - *(Hint: See `docs/coding-standards.md`)*

## Tasks / Subtasks

- [ ] In `src/pyhatchery/cli.py` (or the main orchestrating function):
  - [ ] After all other generation steps (directory creation, file templating from Epic 2, etc.) are complete and successful:
    - [ ] Construct the `project_path` string (e.g., from `final_output_dir / project_name_original` resolved in Story 1.5).
    - [ ] Print the formatted success message including the dynamic `project_name` and `project_path`.
- [ ] Verify (manually or through an E2E test assertion) that PyHatchery's own template source does not lead to a `.git` directory in the generated project (AC1 - this is a one-time check of the template setup).

## Testing Requirements

**Guidance:** Verify implementation against the ACs using the following tests.

- **Unit Tests:**
  - `test_cli.py`:
    - If the success message generation is in a separate utility function, test that function to ensure correct formatting and variable substitution.
  - Location: `tests/unit/`
- **Integration Tests:** (Considered E2E/Acceptance for CLI)
  - In `tests/integration/test_project_generation.py`:
    - After a successful project generation test run (e.g., `pyhatchery new TestProject --no-interactive ...`), capture `stdout`.
    - Verify that the success message is present and correctly formatted, including the project name and path.
    - Verify that the generated `TestProject/` directory does *not* contain a `.git` subdirectory.
- *(Hint: See `docs/testing-strategy.md`)*
