# Story 1.1A: Project Name Validation (PyPI Availability & PEP 8 Convention)

**Status:** Draft

## Goal & Context

**User Story:** As a Python developer, when I provide a project name, I want to be warned if its derived package name is likely already taken on PyPI or if it doesn't follow PEP 8 naming conventions, so I can make an informed decision before proceeding.

**Context:** This story builds upon Story 1.1 where the basic project name is captured. It introduces critical validation steps that provide early feedback to the user, potentially saving them from future naming conflicts or non-standard package names. This story implements parts of PRD FR1A.

## Detailed Requirements

- This story's checks should occur after the project name is input (either via CLI argument from Story 1.1 or during the interactive wizard if it allows name input/confirmation in Story 1.2).
- **PyPI Check:**
      1. Derive the potential PyPI package name from the input project name (e.g., a lowercase, hyphenated slug like "my-project-name").
      2. Query `pypi.org` (using an HTTP client like `requests`) to check if this PyPI package name appears to be taken.
      3. The check must handle network errors (e.g., timeouts, no connection) gracefully. If an error occurs, inform the user that the PyPI check could not be completed and allow them to proceed.
      4. If the name seems to be taken on PyPI, display a non-blocking warning to the user (e.g., "Warning: The name '{pypi_package_name}' might already be taken on PyPI. You may want to choose a different name if you plan to publish this package publicly.").
- **PEP 8 Package Name Check:**
      1. Derive the Python package/module name from the input project name (e.g., an all-lowercase slug with underscores, like "my_project_name", intended for `src/my_project_name` and Python imports).
      2. Validate this derived Python package name against PEP 8 naming conventions for packages (e.g., must be all lowercase, words can be separated by underscores, must be a valid Python identifier).
      3. If the derived name does not conform, display a non-blocking warning (e.g., "Warning: The derived Python package name '{python_package_name}' (from your input '{original_project_name}') does not strictly follow PEP 8 package naming conventions (e.g., 'all_lowercase_with_underscores'). Consider a name that results in a conforming slug for better Python ecosystem compatibility.").
- The user must be able to proceed with the project generation despite any of these warnings. The wizard should ideally ask if they want to proceed or re-enter the name after warnings are shown.
- **Dependency Note:** PyHatchery will require the `requests` library (or a similar HTTP client) as a runtime dependency to implement the PyPI check.

## Acceptance Criteria (ACs)

- AC1: If the derived PyPI package name is likely taken on PyPI, a clear, non-blocking warning is displayed.
- AC2: If the derived Python package name does not conform to PEP 8 package naming conventions, a clear, non-blocking warning is displayed.
- AC3: These checks are performed after the project name is provided, before asking for subsequent details in the interactive wizard.
- AC4: The user can explicitly choose to continue with the current project name despite any warnings, or is offered a chance to re-enter the name.
- AC5: PyPI check failures due to network issues are handled gracefully, inform the user, and do not halt the tool.
- AC6: `requests` (or equivalent) is identified as a new runtime dependency for PyHatchery.

## Technical Implementation Context

**Guidance:** Use the following details for implementation. Refer to the linked `docs/` files for broader context if needed.

- **Relevant Files:**
  - Files to Create: `src/pyhatchery/components/name_service.py`, `src/pyhatchery/components/http_client.py`
  - Files to Modify: `src/pyhatchery/cli.py` (to call the name validation logic), `pyproject.toml` (to add `requests` dependency).
  - _(Hint: See `docs/project-structure.md` for overall layout and component suggestions)_

- **Key Technologies:**
  - Python `>=3.11`
  - `requests` library (for PyPI HTTP GET requests).
  - Standard Python string manipulation and `re` module (for PEP 8 validation).
  - _(Hint: See `docs/tech-stack.md` if available, PRD for dependencies)_

- **API Interactions / SDK Usage:**
  - `GET https://pypi.org/pypi/{package_name}/json`
  - A `200 OK` response implies the name is taken.
  - A `404 Not Found` response implies the name is likely available.
  - Handle other status codes (e.g., `429`, `5xx`) and network exceptions (`requests.exceptions.Timeout`, `requests.exceptions.ConnectionError`) gracefully.
  - _(Hint: See `docs/api-reference.md` for PyPI API details)_

- **Data Structures:**
  - `project_name` (original string from Story 1.1).
  - `pypi_package_name` (derived slug, e.g., "my-project-name").
  - `python_package_name` (derived slug, e.g., "my_project_name").
  - These will become part of the `project_context` (see `docs/data-models.md`). Slug generation logic will be key (revisit Story 2.8 for full slugification requirements, but basic derivation is needed here for validation).
  - _(Hint: See `docs/data-models.md` for `project_context`)_

- **Environment Variables:**
  - Not directly used, but network configuration (proxies) could affect `requests`. This is an advanced case, not for MVP.
  - _(Hint: See `docs/environment-vars.md`)_

- **Coding Standards Notes:**
  - Encapsulate PyPI check logic in `http_client.py`.
  - Encapsulate name derivation (slugging for PyPI/Python package) and PEP 8 validation in `name_service.py`.
  - `name_service.py` should provide functions like `is_pypi_name_taken(pypi_slug)` and `validate_pep8_package_name(python_slug)`.
  - Warnings should be printed to `stderr` or using a dedicated user messaging utility.
  - Ensure non-blocking nature of warnings. Interactive mode (Story 1.2) will handle the "proceed or re-enter" logic. For non-interactive (Story 1.3), warnings are just displayed.
  - _(Hint: See `docs/coding-standards.md` for error handling and style)_

## Tasks / Subtasks

- [ ] Add `requests` to `[project.dependencies]` in `pyproject.toml`.
- [ ] Create `src/pyhatchery/components/http_client.py`:
  - [ ] Implement a function `check_pypi_availability(package_name: str) -> tuple[bool | None, str | None]` which returns (is_taken, error_message). `is_taken` is `None` if check failed.
  - [ ] Handle `requests` exceptions gracefully (timeout, connection error) and return `None, "Network error..."`.
  - [ ] Interpret `200 OK` as taken, `404 Not Found` as available. Other statuses as check failure.
- [ ] Create `src/pyhatchery/components/name_service.py`:
  - [ ] Implement initial slug derivation for PyPI (lowercase, hyphens) and Python package names (lowercase, underscores). (Refer to Story 2.8 for more robust slugification requirements, but a basic version is needed here).
  - [ ] Implement `is_pep8_compliant_package_name(python_package_slug: str) -> bool` using regex or string methods to check for valid Python identifiers and lowercase/underscore convention.
- [ ] In `src/pyhatchery/cli.py` (or a new orchestrating component called by `cli.py`):
  - [ ] After capturing `project_name` (from Story 1.1), call `name_service` to derive PyPI and Python package slugs.
  - [ ] Call `http_client.check_pypi_availability` with the PyPI slug. Display warning if taken or if check fails (AC1, AC5).
  - [ ] Call `name_service.is_pep8_compliant_package_name` with the Python package slug. Display warning if not compliant (AC2).
- [ ] Ensure the flow allows proceeding despite warnings (AC4 part 1 - interactive handling is in Story 1.2).
- [ ] Document `requests` as a runtime dependency.

## Testing Requirements

**Guidance:** Verify implementation against the ACs using the following tests.

- **Unit Tests:**
  - `test_http_client.py`:
    - Mock `requests.get` to simulate different scenarios:
      - PyPI name taken (200 OK response).
      - PyPI name available (404 Not Found response).
      - PyPI API error (e.g., 500 response).
      - Network timeout (`requests.exceptions.Timeout`).
      - Connection error (`requests.exceptions.ConnectionError`).
  - `test_name_service.py`:
    - Test PyPI slug derivation for various inputs.
    - Test Python package slug derivation for various inputs.
    - Test `is_pep8_compliant_package_name` with valid and invalid PEP 8 names.
  - Location: `tests/unit/components/`
- **Integration Tests:**
  - In `tests/integration/test_project_generation.py` (or `test_cli_invocation.py`):
    - Test `pyhatchery new <project_name>` where `<project_name>` is known to be on PyPI. Verify warning message. (Requires mocking `requests.get` at a higher level if testing the full CLI invocation, or testing the internal CLI logic sequence).
    - Test with a project name that yields a non-PEP 8 compliant Python package slug. Verify warning.
    - Test with a project name where PyPI check is mocked to fail (network error). Verify warning and continuation.
- _(Hint: See `docs/testing-strategy.md` for mocking external calls and overall approach)_
