# Story 2.1: `pyproject.toml` Generation and Core Personalization

**Status:** Draft

## Goal & Context

**User Story:** As a Python developer, I want a lean `pyproject.toml` file generated with essential project metadata, dynamic versioning, development dependencies via Hatch's `dependency-groups`, and configurations for Pytest and UV, personalized from my inputs.

**Context:** This is a cornerstone story for Epic 2, implementing PRD FR6 for `pyproject.toml`. It uses the `project_context` (populated in Epic 1) to generate a highly personalized and crucial configuration file. This file dictates build behavior, dependencies, and integrates several tools.

## Detailed Requirements

- Generate a `pyproject.toml` file in the project root.
- Populate `[build-system]`:
  - `requires = ["hatchling>=1.27.0"]`
  - `build-backend = "hatchling.build"`
- Populate `[project]` table with:
  - `name = "<pypi_project_slug>"` (e.g., lowercase, hyphenated)
  - `dynamic = ["version"]`
  - `description = "<project_description_input>"`
  - `authors = [{name = "<author_name_input>", email = "<author_email_input>"}]`
  - `readme = "README.md"`
  - `requires-python = ">=<python_version_preference>"`
  - `license = {text = "<license_choice_input>"}`
  - `classifiers = ["Programming Language :: Python :: 3", "License :: OSI Approved :: MIT License", "Operating System :: OS Independent"]` (adjust Python versions in classifiers based on `requires-python`)
  - `dependencies = []`
- Populate `[project.urls]` with "Homepage", "Releases", "Bug Tracker", "Documentation" pointing to derived GitHub URLs (`https://github.com/<github_username>/<pypi_project_slug>`).
- Populate `[dependency-groups]` (Hatch specific):
  - `dev = ["hatch>=1.14.1", "hatchling>=1.27.0", "pylint>=3.3.7", "pytest>=8.3.5", "pytest-cov>=4.0.0", "ruff>=0.11.8", "uv>=0.7.3"]` (use specified or latest stable versions).
- Populate `[tool.hatch.version]`:
  - `path = "src/<python_package_slug>/__about__.py"`
- Populate `[tool.pytest.ini_options]`:
  - `testpaths = ["tests"]`
  - `python_files = "test_*.py"`
  - `addopts = "-ra -q --cov=src/<python_package_slug> --cov-report=term-missing"`
- Populate `[tool.uv.index]` as specified:
      ```toml
      [tool.uv]
      index = [
        { name = "testpypi", url = "[https://test.pypi.org/simple/](https://test.pypi.org/simple/)", publish-url = "[https://test.pypi.org/legacy/](https://test.pypi.org/legacy/)", explicit = true },
        { name = "pypi", url = "[https://pypi.org/simple/](https://pypi.org/simple/)", publish-url = "[https://upload.pypi.org/legacy/](https://upload.pypi.org/legacy/)", explicit = true },
      ]
      ```
- No `[tool.ruff]` or `[tool.pylint]` sections in `pyproject.toml`.
- Use `jinja2` for templating.

## Acceptance Criteria (ACs)

- AC1: `pyproject.toml` is created in the project root.
- AC2: `[project.name]` correctly reflects the PyPI-slugified version of the project name (`project_slug_pypi`).
- AC3: `[project]` metadata (description, authors, license, readme, requires-python, classifiers, urls) is correctly populated using values from `project_context`.
- AC4: `[project]` lists `dynamic = ["version"]`.
- AC5: `[build-system]` is correctly configured for `hatchling`.
- AC6: `[dependency-groups].dev` lists the specified development tools including `pytest-cov`.
- AC7: `[tool.hatch.version].path` points to `src/<python_package_slug>/__about__.py` (using `project_slug_python`).
- AC8: `[tool.pytest.ini_options]` is configured as specified, including coverage options using `project_slug_python`.
- AC9: `[tool.uv.index]` is configured as specified.
- AC10: `pyproject.toml` does not contain `[tool.ruff]` or `[tool.pylint]` configurations.

## Technical Implementation Context

**Guidance:** Use the following details for implementation. Refer to the linked `docs/` files for broader context if needed.

- **Relevant Files:**
  - Files to Create: `src/pyhatchery/templates/default_project/pyproject.toml.j2`
  - Files to Modify: `src/pyhatchery/components/template_processor.py` (to render this template), `src/pyhatchery/components/project_generator.py` (to call template processor for this file).
  - _(Hint: See `docs/project-structure.md` for template location and component names)_

- **Key Technologies:**
  - Python `>=3.11`
  - `jinja2` for templating.
  - _(Hint: `jinja2` is a core dependency from PRD)_

- **API Interactions / SDK Usage:**
  - Not applicable.

- **Data Structures:**
  - Consumes the `project_context` dictionary (see `docs/data-models.md`). Key fields:
    - `project_slug_pypi`
    - `project_description`
    - `author_name`
    - `author_email`
    - `python_version_preference`
    - `license` (the chosen identifier string, e.g., "MIT")
    - `github_username`
    - `project_slug_python`
  - The list of dev dependencies is fixed for MVP.
  - Python classifiers need to be dynamically adjusted based on `python_version_preference`.
  - _(Hint: See `docs/data-models.md` for `project_context` structure)_

- **Environment Variables:**
  - Not applicable for rendering this template.
  - _(Hint: See `docs/environment-vars.md`)_

- **Coding Standards Notes:**
  - The Jinja2 template (`pyproject.toml.j2`) should be clean and use placeholders like `{{ project_slug_pypi }}`.
  - Logic for classifiers (e.g., if Python 3.10, include "Programming Language :: Python :: 3.10") needs to be in the template or passed into context.
  - `template_processor.py` will load the Jinja2 environment, get the template, and render it with `project_context`.
  - _(Hint: See `docs/coding-standards.md`)_

## Tasks / Subtasks

- [ ] Create the Jinja2 template file `src/pyhatchery/templates/default_project/pyproject.toml.j2`.
  - [ ] Implement `[build-system]` section.
  - [ ] Implement `[project]` section with placeholders for all personalizable fields (`name`, `description`, `authors`, `readme`, `requires-python`, `license`, `classifiers`, `urls`).
  - [ ] Implement dynamic classifier generation based on `{{ python_version_preference }}` (e.g., `Programming Language :: Python :: {{ python_version_preference.split('.')[0] }}`, `Programming Language :: Python :: {{ python_version_preference }}`).
  - [ ] Implement `[dependency-groups].dev` with the specified list of tools.
  - [ ] Implement `[tool.hatch.version]` using `{{ project_slug_python }}`.
  - [ ] Implement `[tool.pytest.ini_options]` using `{{ project_slug_python }}`.
  - [ ] Implement `[tool.uv.index]` section as static text.
- [ ] In `src/pyhatchery/components/template_processor.py` (or if it exists, enhance it):
  - [ ] Ensure it can load and render a given Jinja2 template path with the `project_context`.
- [ ] In `src/pyhatchery/components/project_generator.py` (or main orchestrator):
  - [ ] After base structure is created (Story 1.4), call the `template_processor` to render `pyproject.toml.j2` into the project root, using the fully populated `project_context`.
- [ ] Ensure all specified versions for dev dependencies are accurate.

## Testing Requirements

**Guidance:** Verify implementation against the ACs using the following tests.

- **Unit Tests:**
  - `test_template_processor.py`:
    - Test rendering of `pyproject.toml.j2` with a sample `project_context`.
    - Verify all placeholders are correctly filled.
    - Verify dynamic parts like classifiers are generated correctly based on context.
    - Verify sections that should be absent (like `[tool.ruff]`) are indeed absent.
  - Location: `tests/unit/components/`
- **Integration Tests:** (Considered E2E/Acceptance for CLI)
  - In `tests/integration/test_project_generation.py`:
    - After running `pyhatchery new TestProject --no-interactive ...` with specific inputs:
      - Parse the generated `TestProject/pyproject.toml` (e.g., using `tomllib` since Python 3.11).
      - Assert that `[project.name]` is the correct PyPI slug.
      - Assert authors, description, license text match inputs.
      - Assert `requires-python` matches input.
      - Assert classifiers are correct for the Python version.
      - Assert `[dependency-groups].dev` contains all specified tools.
      - Assert `[tool.hatch.version].path` uses the correct Python package slug.
      - Assert `[tool.pytest.ini_options]` is correct.
      - Assert `[tool.uv.index]` is present and correct.
      - Assert `[tool.ruff]` and `[tool.pylint]` are not present.
- _(Hint: See `docs/testing-strategy.md` and `docs/data-models.md` for `project_context`)_
