# PyHatchery Data Models

This document describes the key data structures used within PyHatchery, primarily focusing on the context object passed to the Jinja2 templating engine for project generation.

## 1. Core Application Entities / Domain Objects

The most significant "data model" within PyHatchery is the `project_context` object. This object (typically a Python dictionary) aggregates all user-provided and derived information necessary to render the project templates.

### `project_context` Object

* **Description:** A dictionary containing all variables made available to the Jinja2 templates when generating a new project. This context is dynamically built by the `ContextBuilder` component based on user inputs, derived values, and defaults.
* **Schema / Structure (Conceptual):**
    While Python dictionaries are flexible, the `project_context` is expected to contain the following keys:

    ```python
    # Conceptual representation of the project_context dictionary keys and their typical data types
    project_context = {
        "project_name": str,  # Original project name as input by the user (e.g., "My Awesome Project")
        "project_slug_pypi": str,  # PyPI-friendly slug (e.g., "my-awesome-project")
        "project_slug_python": str,  # Python module/package friendly slug (e.g., "my_awesome_project")
        "author_name": str,  # Author's name (e.g., "Jane Doe")
        "author_email": str,  # Author's email (e.g., "jane.doe@example.com")
        "github_username": str,  # Author's GitHub username (e.g., "janedoe")
        "project_description": str,  # A short description of the project
        "license": str,  # Chosen license identifier (e.g., "MIT", "Apache-2.0")
        "python_version": str,  # Target Python version for the generated project (e.g., "3.10", "3.11")
        "python_version_short": str, # Short form of Python version (e.g., "310", "311") for use in tool configs like ruff.toml target-version
        "current_year": str,  # Current calendar year (e.g., "2025") for license personalization
        "pyhatchery_version": str, # Version of PyHatchery used to generate the project (e.g., "0.1.0")
        "output_dir_name": str, # The name of the root directory for the generated project (often same as project_name or a slug)
        # Potentially other useful variables for advanced templating if needed
    }
    ```

* **Validation Rules:**
  * Most string fields derived from user input (like `project_name`, `author_name`, etc.) are expected to be non-empty if they are required by the wizard or non-interactive flags.
  * `project_slug_pypi` and `project_slug_python` are generated by specific logic and validated (PEP 8, PyPI availability check warnings) by the `ProjectNameService`.
  * `license` will be one of the predefined choices offered to the user.
  * `python_version` will be one of the supported Python versions offered.

## 2. Internal Project Template Structure (Conceptual)

The files located in `src/pyhatchery/templates/default_project/` (and its subdirectories) are Jinja2 templates. These templates form the blueprint of the projects generated by PyHatchery.

* **Description:** These are text files (e.g., `pyproject.toml.j2`, `README.md.j2`) that contain standard boilerplate code/text along with Jinja2 placeholders and control structures (e.g., `{{ project_name }}`, `{% if license == "MIT" %}`).
* **Data Consumption:** These templates consume the variables provided in the `project_context` object described above. For instance, a `LICENSE.j2` template would use `{{ author_name }}` and `{{ current_year }}`.
* **File Naming:** Template files often use a `.j2` suffix (or similar convention like `.template`) to distinguish them from actual output files, though the final generated files will have the target names (e.g., `README.md`). Directory names within the template structure can also use Jinja2 variables (e.g., `src/{{ project_slug_python }}/`) to allow for dynamic naming based on the context.
* **Example Placeholders in Templates:**
  * `{{ project_name }}`
  * `{{ project_slug_pypi }}`
  * `{{ project_slug_python }}`
  * `{{ author_name }}`
  * `{{ author_email }}`
  * `{{ github_username }}`
  * `{{ license }}`
  * `{{ python_version }}`
  * `{{ python_version_short }}`
  * `{{ current_year }}`

This conceptual model ensures that the `TemplateProcessor` component can systematically render a complete, personalized project structure using the `project_context`.

## 3. API Payload Schemas

Not applicable, as PyHatchery does not expose an API for external consumption. It only consumes the PyPI API (see `docs/api-reference.md`).

## 4. Database Schemas

Not applicable, as PyHatchery does not use a database.

## 5. State File Schemas

Not applicable, as PyHatchery does not rely on state files for its operation between invocations.

## Change Log

| Change        | Date       | Version | Description                                                        | Author      |
| ------------- | ---------- | ------- | ------------------------------------------------------------------ | ----------- |
| Initial draft | 2025-05-09 | 0.1     | Initial draft defining project_context and template structure concept. | 3-Architect |
