# Epic 4: CI/CD Workflow Automation

**Goal:** Create and include pre-configured GitHub Actions workflow files for automated testing on pushes/PRs (utilizing Hatch scripts) and (optionally enabled) package publishing to TestPyPI/PyPI, with clear instructions for activation in the generated project's `README.md`.

## Story List

### Story 4.1: GitHub Actions Workflow for Automated Testing

- **User Story / Goal:** As a Python developer, I want a GitHub Actions workflow that automatically runs tests and linters (using Hatch scripts) on every push to `main`/`develop` and on pull requests, across multiple Python versions.
- **Detailed Requirements:**
  - Create a workflow file (e.g., `.github/workflows/ci.yml` or `tests.yml`).
  - The workflow should trigger on:
    - `push` to `main` and `develop` branches.
    - `pull_request` targeting `main` and `develop` branches.
  - It should set up a matrix strategy to test against multiple Python versions (e.g., 3.10, 3.11, 3.12, matching project's supported range).
  - Steps should include:
    - Checking out the code (`actions/checkout`).
    - Setting up Python (`actions/setup-python`) for each matrix version.
    - Installing `uv` (via the official `astral-sh/setup-uv@v6`).
    - Creating a virtual environment and installing dependencies using `uv pip install -e .[dev]`.
    - Running quality checks using Hatch scripts: `hatch run check-all` (which includes linting and tests with coverage).
    - Optionally, uploading coverage reports (e.g., to Codecov using `codecov/codecov-action` or as a GitHub artifact).
- **Acceptance Criteria (ACs):**
  - AC1: A `ci.yml` (or similar) workflow file is created in `.github/workflows/`.
  - AC2: The workflow triggers correctly on specified push and pull_request events.
  - AC3: Tests and linters are run across a matrix of defined Python versions.
  - AC4: Workflow steps correctly checkout code, set up Python and `uv`, install dependencies, and execute `hatch run check-all`.
  - AC5: The workflow successfully passes if `hatch run check-all` passes.
  - AC6: (Optional but recommended) Coverage data is collected and can be uploaded/reported.

### Story 4.2: GitHub Actions Workflow for Publishing to PyPI/TestPyPI

- **User Story / Goal:** As a Python developer, I want a GitHub Actions workflow that can build and publish my package to TestPyPI (on `develop` branch pushes) and PyPI (on `main` branch pushes or tag pushes), which I can enable easily.
- **Detailed Requirements:**
  - Create a workflow file (e.g., `.github/workflows/publish.yml`).
  - The workflow should trigger on:
    - `push` to `develop` (for TestPyPI).
    - `push` to `main` (for PyPI).
    - `push` to tags matching `v*.*.*` (for PyPI).
  - Implement a condition to check an environment variable or workflow input `ENABLE_PUBLISHING` (defaulting to `false`). The workflow should only proceed with publishing if this is explicitly set to `true` via GitHub repository variable/secret.
  - Steps should include:
    - Checking out the code.
    - Setting up Python.
    - Installing `hatch` (done simply by running `uv`; e.g. `run: uv sync --dev`).
    - Building the sdist and wheel using `hatch run build`.
    - Publishing to TestPyPI using `hatch publish -r testpypi --user __token__ --password ${{ secrets.TEST_PYPI_API_TOKEN }}` (if `ENABLE_PUBLISHING` is true and on `develop` branch and `TEST_PYPI_API_TOKEN` secret exists).
    - Publishing to PyPI using `uv publish --index pypi [...]` (if `ENABLE_PUBLISHING` is true and on `main`/tag and `PYPI_API_TOKEN` secret exists).
  - The workflow should clearly indicate success or failure of publishing.
- **Acceptance Criteria (ACs):**
  - AC1: A `publish.yml` (or similar) workflow file is created in `.github/workflows/`.
  - AC2: The workflow triggers correctly on specified branch/tag push events.
  - AC3: Publishing steps are skipped by default (when `ENABLE_PUBLISHING` is not `true` or secrets are missing).
  - AC4: If `ENABLE_PUBLISHING` is true and conditions met, the workflow attempts to publish to TestPyPI using `TEST_PYPI_API_TOKEN`.
  - AC5: If `ENABLE_PUBLISHING` is true and conditions met, the workflow attempts to publish to PyPI using `PYPI_API_TOKEN`.
  - AC6: Build artifacts (sdist, wheel) are correctly generated by `hatch run build` before publishing attempt.

### Story 4.3: Documentation for Enabling Publishing Workflows

- **User Story / Goal:** As a Python developer, I want clear instructions in my project's `README.md` on how to enable the publishing workflow and set up the necessary PyPI API token secrets in GitHub.
- **Detailed Requirements:**
  - The generated `README.md` must include a section detailing:
    - How the publishing workflow (`publish.yml`) works (triggers, conditions).
    - How to enable publishing by setting the GitHub repository variable `ENABLE_PUBLISHING` to `true`.
    - Step-by-step instructions on how to generate API tokens on PyPI and TestPyPI (scoped to the project if possible).
    - Step-by-step instructions on how to add these tokens as `PYPI_API_TOKEN` and `TEST_PYPI_API_TOKEN` secrets in the GitHub repository settings.
    - A warning about securing API tokens.
- **Acceptance Criteria (ACs):**
  - AC1: `README.md` contains a dedicated section for "Publishing to PyPI".
  - AC2: Instructions clearly explain the `ENABLE_PUBLISHING` repository variable mechanism.
  - AC3: Instructions accurately guide the user through creating PyPI/TestPyPI tokens.
  - AC4: Instructions accurately guide the user through adding these tokens as GitHub secrets with the correct names.
  - AC5: The documentation is easy to understand and follow.
